/* 
 * Copyright (C) 1997, 1998 Defence Research Establishment, Dept. of Comm.
 * Systems, FOA 72, Sweden.
 * Currently maintained by Christian Jnsson FOA 72 <chj@lin.foa.se>.
 *
 * This file is part of ETFTP version 1.1.3.
 *
 *   ETFTP is free software; you can redistribute it and/or modify it
 *   under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   ETFTP is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with ETFTP; see the file GPL.  If not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *   Boston, MA 02111-1307, USA.
 *
 * Code:
 *
 *----------------------------------------------------------------------
 * Filenamne:   defs.h
 * Module:      etftp
 * Created:     0.1, 970428, Torbjrn Frostemark
 * Revised:     1.1, 980219, Torbjrn Frostemark
 * 
 * Description: Definitions needed for ETFTP client and server
 * Remarks:     -
 *----------------------------------------------------------------------
 */

#ifndef _DEFS
#define _DEFS

#ifdef CLIENT
#define CLIENT_OR_SERVER
#endif
#ifdef SERVER
#define CLIENT_OR_SERVER
#endif

#ifdef CLIENT_OR_SERVER

/*
 * ETFTP client and server stuff
 */

#define ETFTP_VERSION	"1.1.3"

/* max size of data per packet to send or rcv 512 is specified by the RFC */
#define MAXDATA           512
/* max filename length */
#define MAXFILENAME       128
/* max host name length */
#define MAXHOSTNAME       128
/* max command line length */
#define MAXLINE           512
/* max token length */
#define MAXTOKEN          128

/*
 * Define the etftp error codes.
 * These are transmitted in PT_REFUSED, PT_QUIT, and PT_ABORT
 * in the reason field.
 */

/* File not found */
#define ERR_NOFILE      "File not found!"
/* Access denied */
#define ERR_REFUSED     "Server refused connection!"
/* Disk full or allocation exceeded */
#define ERR_NOSPACE     "Disk full!"
/* Illegal etftp operation */
#define ERR_BADOP       "Illegal action!"
/* Unknown TID (port#) */
#define ERR_BADID       "Bad port number!"
/* File already exists */
#define ERR_FILE        "File already exists!"
/* No such user */
#define ERR_NOUSER      "Bad user ID!"

#endif /* CLIENT_OR_SERVER */

/*
 * Declarations used in all three components
 */

/* Define the etftp packet types numbers according to RFC998. */

/* open request by client containing suggested parameters */
#define PT_OPEN                 0
/* positive answer by server containing agreed parameters */
#define PT_RESPONSE             1
/* The package type below is NOT used in ETFTP, but in NETBLT */
#define PT_KEEPALIVE            2
/* request for emergency disconnect */
#define PT_QUIT                 3
/* acknowledge QUIT */
#define PT_QUITACK              4
/* disconnect, no questions asked */
#define PT_ABORT                5
/* data */
#define PT_DATA                 6
/* last data in buffer */
#define PT_LDATA                7
/* acknowledge new parameters from PT_CONTROL_OK */
#define PT_NULL_ACK             8
/* The package type below has three subtypes */
#define PT_CONTROL              9
/* sent by reciever when ready to recieve */
#define PST_GO                  0
/* sent by reciever after successful buffer, maybe with new parameters */
#define PST_OK                  1
/* sent by reciever containing list with packets to be resent */
#define PST_RESEND              2
/* negative answer by server */
#define PT_REFUSED              10
/* acknowledge last NULL_ACK before disconnect*/
#define PT_DONE                 11

/* minimum type value */
#define PT_MIN  0
/* maximum type value */
#define PT_MAX  11

/*
 * Define macros to load and store 2-byte integers, since these are
 * used in the ETFTP headers for opcodes, block numbers and error
 * numbers.  These macros handle the conversion between host format
 * and network byte ordering.
 */

/* Macros for storing values in network order */
#define stlong(lval, addr) (*( (unsigned long *)(addr) ) = htonl(lval) )
#define stshort(sval, addr)  (*( (unsigned short *)(addr) ) = htons(sval) )
#define ldshort(addr)           ( ntohs (*( (unsigned short *)(addr) ) ) )
#define ldlong(addr)           ( ntohl (*( (unsigned long *)(addr) ) ) )

#ifdef  lint                    /* hush up lint */
#undef  ldshort
#undef  stshort
#undef  ldlong
#undef  stlong

extern short    ldshort();	/* an exeption */
#endif  /* lint */


/*
 * Debug macros, based on the trace flag (-v command line argument,
 * or "verbose" command in client).
 */

#ifdef SERVER
#define DEBUG0(fmt)             if (verboseFlag) { \
                                        fprintf(debugFD, fmt); \
                                        fputc('\n', debugFD); \
                                        fflush(debugFD); \
                                }

#define DEBUG1(fmt, arg1)       if (verboseFlag) { \
                                        fprintf(debugFD, fmt, arg1); \
                                        fputc('\n', debugFD); \
                                        fflush(debugFD); \
                                }

#define DEBUG2(fmt, arg1, arg2) if (verboseFlag) { \
                                        fprintf(debugFD, fmt, arg1, arg2); \
                                        fputc('\n', debugFD); \
                                        fflush(debugFD); \
                                }

#define DEBUG3(fmt, arg1, arg2 ,arg3) if (verboseFlag) { \
                                        fprintf(debugFD, fmt, arg1, arg2, arg3); \
                                        fputc('\n', debugFD); \
                                        fflush(debugFD); \
                                }
#define DEBUG4(fmt, arg1, arg2 ,arg3, arg4) if (verboseFlag) { \
                                        fprintf(debugFD, fmt, arg1, arg2, arg3, arg4); \
                                        fputc('\n', debugFD); \
                                        fflush(debugFD); \
                                }
#define DEBUG5(fmt, arg1, arg2 ,arg3, arg4, arg5) if (verboseFlag) { \
                                        fprintf(debugFD, fmt, arg1, arg2, arg3, arg4, arg5); \
                                        fputc('\n', deubgFD); \
                                        fflush(debugFD); \
                                }
#else /* CLIENT */
#define DEBUG0(fmt)             if (verboseFlag) { \
                                        fprintf(stderr, fmt); \
                                        fputc('\n', stderr); \
                                        fflush(stderr); \
                                }

#define DEBUG1(fmt, arg1)       if (verboseFlag) { \
                                        fprintf(stderr, fmt, arg1); \
                                        fputc('\n', stderr); \
                                        fflush(stderr); \
                                }

#define DEBUG2(fmt, arg1, arg2) if (verboseFlag) { \
                                        fprintf(stderr, fmt, arg1, arg2); \
                                        fputc('\n', stderr); \
                                        fflush(stderr); \
                                }

#define DEBUG3(fmt, arg1, arg2 ,arg3) if (verboseFlag) { \
                                        fprintf(stderr, fmt, arg1, arg2, arg3); \
                                        fputc('\n', stderr); \
                                        fflush(stderr); \
                                }
#define DEBUG4(fmt, arg1, arg2 ,arg3, arg4) if (verboseFlag) { \
                                        fprintf(stderr, fmt, arg1, arg2, arg3, arg4); \
                                        fputc('\n', stderr); \
                                        fflush(stderr); \
                                }
#define DEBUG5(fmt, arg1, arg2 ,arg3, arg4, arg5) if (verboseFlag) { \
                                        fprintf(stderr, fmt, arg1, arg2, arg3, arg4, arg5); \
                                        fputc('\n', stderr); \
                                        fflush(stderr); \
                                }
#endif /* CLIENT */

#define CLIDISP(fmt) if (interactive) { \
                                        fprintf(stdout, fmt ); \
                                        fflush(stdout); \
                                }

#define CLIDISP1(fmt, arg1) if (interactive) { \
                                        fprintf(stdout, fmt, arg1 ); \
                                        fflush(stdout); \
                                }

/* Common Header package fields */
#define PF_CHECKSUM(pk) (256*(unsigned char)pk[0]+(unsigned char)pk[1])
#define PF_VERSION(pk) ((unsigned char)pk[2])
#define PF_TYPE(pk) ((unsigned char)pk[3])
#define PF_LENGTH(pk) (256*(unsigned char)pk[4]+(unsigned char)pk[5])
#define PF_LOCALPORT(pk) (256*(unsigned char)pk[6]+(unsigned char)pk[7])
#define PF_REMOTEPORT(pk) (256*(unsigned char)pk[8]+(unsigned char)pk[9])

/* OPEN and RESPONSE specific fields */
#define PF_CONNECT_ID(pk) (long)(256*256*256*(unsigned char)pk[12]+256*256*(unsigned char)pk[13]+256*(unsigned char)pk[14]+(unsigned char)pk[15])
#define PF_BUFSZ(pk) (long)(256*256*256*(unsigned char)pk[16]+256*256*(unsigned char)pk[17]+256*(unsigned char)pk[18]+(unsigned char)pk[19])
#define PF_TRXSZ(pk) (long)(256*256*256*(unsigned char)pk[20]+256*256*(unsigned char)pk[21]+256*(unsigned char)pk[22]+(unsigned char)pk[23])
#define PF_DATAPACKSZ(pk) (256*(unsigned char)pk[24]+(unsigned char)pk[25])
#define PF_BURSTSZ(pk) (256*(unsigned char)pk[26]+(unsigned char)pk[27])
#define PF_BURSTRATE(pk) (256*(unsigned char)pk[28]+(unsigned char)pk[29])
#define PF_DEATHTIMER(pk) (256*(unsigned char)pk[30]+(unsigned char)pk[31])
#define PF_OPENMODE(pk) ((unsigned char)(unsigned char)pk[33] & 0x1)
#define PF_TRMODE(pk) ((unsigned char)(unsigned char)pk[33] & 0x4)
#define PF_RENEGOTIATE(pk) ((unsigned char)(unsigned char)pk[33] & 0x8)
#define PF_MAXOUTBUF(pk) (256*(unsigned char)pk[34]+(unsigned char)pk[35])
#define PF_RADIODELAY(pk) (256*(unsigned char)pk[36]+(unsigned char)pk[37])
#define PF_FILENAME(pk) (&(pk[40]))

/* QUIT, ABORT, REFUSED specific fields OBS!! extracts a (char *) */
#define PF_REASON(pk) (&(pk[12]))

/* DATA and LDATA specific fields */
#define PF_BUFNUM(pk) (long)(256*256*256*(unsigned char)pk[12]+256*256*(unsigned char)pk[13]+256*(unsigned char)pk[14]+(unsigned char)pk[15])
#define PF_TOPNUM(pk) (256*(unsigned char)pk[16]+(unsigned char)pk[17])
#define PF_PACKNUM(pk) (256*(unsigned char)pk[18]+(unsigned char)pk[19])
#define PF_DATACHKSUM(pk) (256*(unsigned char)pk[20]+(unsigned char)pk[21])
#define PF_LASTFLAG(pk) ((unsigned char)(unsigned char)pk[23] & 0x1)

/* NULL_ACK specific fields OBS!! different locations from OPEN and RESPONSE */
#define PF_TOPNUM_ACK(pk) (256*(unsigned char)pk[12]+(unsigned char)pk[13])
#define PF_BURSTSZ_ACK(pk) (256*(unsigned char)pk[14]+(unsigned char)pk[15])
#define PF_BURSTRATE_ACK(pk) (256*(unsigned char)pk[16]+(unsigned char)pk[17])
#define PF_PACKSZ_ACK(pk) (256*(unsigned char)pk[18]+(unsigned char)pk[19])

/* CONTROL_OK, CONTROL_GO and CONTROL_RESEND specific fields */
#define PF_SUBTYPE(pk) ((unsigned char)pk[12])
#define PF_SEQNUM(pk) (256*(unsigned char)pk[14]+(unsigned char)pk[15])
#define PF_BUFNUM_REQ(pk) (long)(256*256*256*(unsigned char)pk[16]+256*256*(unsigned char)pk[17]+256*(unsigned char)pk[18]+(unsigned char)pk[19])
#define PF_NUMOFPK(pk) (256*(unsigned char)pk[20]+(unsigned char)pk[21])

/* CONTROL_OK and CONTROL_GO specific fields */
#define PF_BURSTSZ_REQ(pk) (256*(unsigned char)pk[20]+(unsigned char)pk[21])
#define PF_BURSTRATE_REQ(pk) (256*(unsigned char)pk[22]+(unsigned char)pk[23])
#define PF_CTRLTIMER(pk) (256*(unsigned char)pk[24]+(unsigned char)pk[25])
#define PF_PACKSZ_REQ(pk) (256*(unsigned char)pk[26]+(unsigned char)pk[27])
/* CONTROL_GO exceeding CONTROL_GO */
#define PF_SUBTYPE2(pk) ((unsigned char)pk[28])

/* CONTROL_RESEND specific fields */
#define PF_PACKCNT(pk) (256*(unsigned char)pk[20]+(unsigned char)pk[21])
#define PF_PACKLIST(pk) (&(pk[24]))

/* Values for modType. */

/* ascii == netascii */
#define MODE_ASCII      0
/* binary == octet */
#define MODE_BINARY     1

/* Max value definitions (twice the default). */

/* max block size in bytes */
#define MAXBLOCKSZ	1448
/* max buffer size in blocks */
#define MAXBUFSZ	512
/* max burst size in packets */
#define MAXBURSTSZ	16
/* max packet length in bytes */
#define MAXPACKETSZ	1472
/* name of the service */
#define ETFTP_SERVICE	"etftp"
/* max buffer size in blocks (?) */
#define	MAXBUFF		2048

/* maximum length of init file */
#define MAXINITLEN	2048
/* number of integers to read from file */
#define NUMINT		7
/* number of floats to read from file */
#define NUMFLOAT	4

/* Some useful constants */
#define MODE_PUT	0
#define MODE_GET	1

#define FALSE		0
#define TRUE		1

#define DONE		0
#define NOT_DONE	1

#define NULL_BYTE	((char) 0)
#define DEATHTIME	0

#define DESTROY		0
#define FORWARD		1

#define THOUSAND	1000

#endif /* _ETFTP */


