/* 
 * Copyright (C) 1997, 1998 Defence Research Establishment, Dept. of Comm.
 * Systems, FOA 72, Sweden.
 * Currently maintained by Christian Jnsson FOA 72 <chj@lin.foa.se>.
 *
 * This file is part of ETFTP version 1.1.3.
 *
 *   ETFTP is free software; you can redistribute it and/or modify it
 *   under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   ETFTP is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with ETFTP; see the file GPL.  If not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *   Boston, MA 02111-1307, USA.
 *
 * Code:
 *
 *----------------------------------------------------------------------
 * Filenamne:   tron.h
 * Class:       Tron
 * Created:     0.1, 970420, Per-Erik Forssn
 * Revised:     1.1, 980219, Per-Erik Forssn
 *
 * Public Attributes:
 *              -
 * Public Methods:
 *              TRONStart(burstSize,burstRate,dataSize,timeVal);
 *              TRONReset(bufSize);
 *              double TRONUpdate(packNum,packSize);
 *              int TRONDecide(*burstSize,*burstRate,*dataSize,*timeVal);
 *              TRONList *TRONResend();
 *
 * Description: Used by the reciever to calculate new transfer-parameters
 *		based on how sucessful the previous buffer transfer was.
 *		Also calculates average throughput.
 *
 * Remarks:     -
 *----------------------------------------------------------------------
 */

#ifndef _TRON
#define _TRON

#include "etftp.h"

/*
 * Methods available in tron.c
 */
extern void TRONStart(unsigned short burstSize, unsigned short burstRate,
		      unsigned short blockSize);
extern void TRONReset(short bufferSize, int fileSize);
extern double TRONUpdate(short packNum, int packSize);
extern int TRONDecide(unsigned short *burstSize, unsigned short *burstRate,
		      unsigned short *blockSize);
extern TRONList *TRONResend();

/*
 * Private macro (halve packet size in multiples of four.)
 */

#define TRONHalve(dataSize) ((dataSize/8)*4)

#endif /* _TRON */




